@@ -234,6 +234,7 @@ typedef struct {
 	unsigned short max_write_idle;
 	unsigned short use_xattr;
 	unsigned short follow_symlink;
+	unsigned short range_requests;
 	
 	/* debug */
 	
@@ -293,6 +294,11 @@ typedef struct {
 	struct timeval start_tv;
 	
 	size_t request_count;        /* number of requests handled in this connection */
+	size_t loops_per_request;    /* to catch endless loops in a single request
+				      * 
+				      * used by mod_rewrite, mod_fastcgi, ... and others
+				      * this is self-protection
+				      */
 	
 	int fd;                      /* the FD for this connection */
 	int fde_ndx;                 /* index for the fdevent-handler */
@@ -402,13 +408,15 @@ typedef struct {
 typedef struct {
 	unsigned short port;
 	buffer *bindhost;
-	buffer *error_logfile;
+	
+	buffer *errorlog_file;
+	unsigned short errorlog_use_syslog;
+	
 	unsigned short dont_daemonize;
 	buffer *changeroot;
 	buffer *username;
 	buffer *groupname;
 	
-	buffer *license;
 	buffer *pid_file;
 	
 	buffer *event_handler;
@@ -450,13 +458,17 @@ typedef struct {
 typedef struct {
 	server_socket_array srv_sockets;
 	
-	int log_error_fd;
-	int log_using_syslog;
+	/* the errorlog */
+	int errorlog_fd;
+	enum { ERRORLOG_STDERR, ERRORLOG_FILE, ERRORLOG_SYSLOG } errorlog_mode;
+	buffer *errorlog_buf;
+	
 	fdevents *ev, *ev_ins;
 	
 	buffer_plugin plugins;
 	void *plugin_slots;
 	
+	/* counters */
 	int con_opened;
 	int con_read;
 	int con_written;
@@ -472,7 +484,6 @@ typedef struct {
 	/* buffers */
 	buffer *parse_full_path;
 	buffer *response_header;
-	buffer *error_log;
 	buffer *response_range;
 	buffer *tmp_buf;
 	
