@@ -76,6 +76,8 @@ static int config_insert(server *srv) {
 		{ "dir-listing.external-css",    NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },  /* 40 */
 		
 		{ "dir-listing.encoding",        NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },  /* 41 */
+		{ "server.errorlog-use-syslog",  NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_SERVER },     /* 42 */
+		{ "server.range-requests",       NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION }, /* 43 */
 		
 		{ "server.host",                 "use server.bind instead", T_CONFIG_DEPRECATED, T_CONFIG_SCOPE_UNSET },
 		{ "server.docroot",              "use server.document-root instead", T_CONFIG_DEPRECATED, T_CONFIG_SCOPE_UNSET },
@@ -92,7 +94,7 @@ static int config_insert(server *srv) {
 	
 	/* 0 */
 	cv[0].destination = srv->srvconf.bindhost;
-	cv[1].destination = srv->srvconf.error_logfile;
+	cv[1].destination = srv->srvconf.errorlog_file;
 	cv[3].destination = srv->srvconf.changeroot;
 	cv[4].destination = srv->srvconf.username;
 	cv[5].destination = srv->srvconf.groupname;
@@ -107,6 +109,8 @@ static int config_insert(server *srv) {
 	cv[36].destination = &(srv->srvconf.log_request_header_on_error);
 	cv[37].destination = &(srv->srvconf.log_state_handling);
 	
+	cv[42].destination = &(srv->srvconf.errorlog_use_syslog);
+	
 	srv->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
 
 	assert(srv->config_storage);
@@ -139,6 +143,7 @@ static int config_insert(server *srv) {
 		s->follow_symlink = 1;
 		s->kbytes_per_second = 0;
 		s->allow_http11  = 1;
+		s->range_requests = 1;
 		s->global_kbytes_per_second = 0;
 		s->global_bytes_per_second_cnt = 0;
 		s->global_bytes_per_second_cnt_ptr = &s->global_bytes_per_second_cnt;
@@ -179,6 +184,7 @@ static int config_insert(server *srv) {
 		cv[39].destination = &(s->hide_dotfiles);
 		cv[40].destination = s->dirlist_css;
 		cv[41].destination = s->dirlist_encoding;
+		cv[43].destination = &(s->range_requests);
 		
 		srv->config_storage[i] = s;
 	
@@ -225,6 +231,8 @@ int config_setup_connection(server *srv, connection *con) {
 	PATCH(log_request_handling);
 	PATCH(log_file_not_found);
 	
+	PATCH(range_requests);
+	
 	return 0;
 }
 
@@ -250,6 +258,8 @@ int config_patch_connection(server *srv, connection *con, const char *stage, siz
 				PATCH(document_root);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("server.dir-listing"))) {
 				PATCH(dir_listing);
+			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("server.range-requests"))) {
+				PATCH(range_requests);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("dir-listing.hide-dotfiles"))) {
 				PATCH(hide_dotfiles);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("dir-listing.external-css"))) {
