@@ -8,6 +8,9 @@
 
 int f_crypto_md5(lua_State *L);
 int f_file_mtime(lua_State *L);
+int f_file_isreg(lua_State *L);
+int f_dir_files(lua_State *L);
+
 int f_memcache_exists(lua_State *L);
 int f_memcache_get_string(lua_State *L);
 int f_memcache_get_long(lua_State *L);
