@@ -1,6 +1,7 @@
 #include <assert.h>
 #include <stdio.h>
 #include <errno.h>
+#include <time.h>
 
 #include "mod_cml.h"
 #include "mod_cml_funcs.h"
@@ -94,8 +95,6 @@ static int lua_to_c_is_table(lua_State *L, const char *varname) {
 }
 
 static int c_to_lua_push(lua_State *L, int tbl, const char *key, size_t key_len, const char *val, size_t val_len) {
-	fprintf(stderr, "%*s -> %*s\n", key_len, key, val_len, val);
-	
 	lua_pushlstring(L, key, key_len);
 	lua_pushlstring(L, val, val_len);
 	lua_settable(L, tbl);
@@ -166,6 +165,8 @@ int cache_parse_lua(server *srv, connection *con, plugin_data *p, buffer *fn) {
 	/* register functions */
 	lua_register(L, "md5", f_crypto_md5);
 	lua_register(L, "file_mtime", f_file_mtime);
+	lua_register(L, "file_isreg", f_file_isreg);
+	lua_register(L, "dir_files", f_dir_files);
 	
 #ifdef HAVE_MEMCACHE_H
 	lua_pushliteral(L, "memcache_get_long");
@@ -184,11 +185,11 @@ int cache_parse_lua(server *srv, connection *con, plugin_data *p, buffer *fn) {
 	lua_settable(L, LUA_GLOBALSINDEX);
 #endif
 	/* register CGI environment */
-	lua_pushstring(L, "request");
+	lua_pushliteral(L, "request");
 	lua_newtable(L);
 	lua_settable(L, LUA_GLOBALSINDEX);
 	
-	lua_pushstring(L, "request");
+	lua_pushliteral(L, "request");
 	header_tbl = lua_gettop(L);
 	lua_gettable(L, LUA_GLOBALSINDEX);
 	
@@ -204,18 +205,26 @@ int cache_parse_lua(server *srv, connection *con, plugin_data *p, buffer *fn) {
 	c_to_lua_push(L, header_tbl, CONST_STR_LEN("BASEURL"), CONST_BUF_LEN(p->baseurl));
 	
 	/* register GET parameter */
-	lua_pushstring(L, "get");
+	lua_pushliteral(L, "get");
 	lua_newtable(L);
 	lua_settable(L, LUA_GLOBALSINDEX);
 	
-	lua_pushstring(L, "get");
+	lua_pushliteral(L, "get");
 	header_tbl = lua_gettop(L);
 	lua_gettable(L, LUA_GLOBALSINDEX);
 	
+	
 	buffer_copy_string_buffer(b, con->uri.query);
 	split_query_string(L, header_tbl, b);
 	buffer_reset(b);
 	
+	lua_pushliteral(L, "CACHE_HIT");
+	lua_pushnumber(L, 0);
+	lua_settable(L, LUA_GLOBALSINDEX);
+	
+	lua_pushliteral(L, "CACHE_MISS");
+	lua_pushnumber(L, 1);
+	lua_settable(L, LUA_GLOBALSINDEX);
 	
 	/* load lua program */
 	if (lua_load(L, load_file, &rm, fn->ptr) || lua_pcall(L,0,1,0)) {
@@ -249,11 +258,21 @@ int cache_parse_lua(server *srv, connection *con, plugin_data *p, buffer *fn) {
 		
 		struct stat st;
 		int curelem;
+		time_t mtime = 0;
 		
 		lua_pushstring(L, "output_include");
 		
 		curelem = lua_gettop(L);
 		lua_gettable(L, LUA_GLOBALSINDEX);
+
+		/* HOW-TO build a etag ?
+		 * as we don't just have one file we have to take the stat() 
+		 * from all base files, merge them and build the etag from
+		 * it later.
+		 * 
+		 * The mtime of the content is the mtime of the freshest base file
+		 * 
+		 * */
 		
 		lua_pushnil(L);  /* first key */
 		while (lua_next(L, curelem) != 0) {
@@ -287,6 +306,7 @@ int cache_parse_lua(server *srv, connection *con, plugin_data *p, buffer *fn) {
 					}
 				} else {
 					chunkqueue_append_file(con->write_queue, b, 0, st.st_size);
+					if (st.st_mtime > mtime) mtime = st.st_mtime;
 				}
 			} else {
 				/* not a string */
@@ -303,6 +323,21 @@ int cache_parse_lua(server *srv, connection *con, plugin_data *p, buffer *fn) {
 		
 		if (ret == 0) {
 			con->file_finished = 1;
+			
+			/* no Last-Modified specified */
+			if (mtime && NULL == array_get_element(con->response.headers, "Last-Modified")) {
+				char timebuf[sizeof("Sat, 23 Jul 2005 21:20:01 GMT")];
+		
+				strftime(timebuf, sizeof(timebuf), "%a, %d %b %Y %H:%M:%S GMT", gmtime(&mtime));
+				
+				response_header_overwrite(srv, con, CONST_STR_LEN("Last-Modified"), timebuf, sizeof(timebuf) - 1);
+			}
+			
+			if (http_response_handle_cachable(srv, con, mtime)) {
+				/* ok, the client already has our content, 
+				 * no need to send it again */
+				chunkqueue_reset(con->write_queue);
+			}
 		} else {
 			chunkqueue_reset(con->write_queue);
 		}
