@@ -62,8 +62,7 @@ int http_response_write_basic_header(server *srv, connection *con) {
 		}
 	}
 	
-	if (con->request.http_version == HTTP_VERSION_1_1 &&
-	    (con->parsed_response & HTTP_DATE) == 0) {
+	if ((con->parsed_response & HTTP_DATE) == 0) {
 		/* HTTP/1.1 requires a Date: header */
 		BUFFER_APPEND_STRING_CONST(b, "\r\nDate: ");
 	
@@ -93,7 +92,12 @@ int http_response_write_basic_header(server *srv, connection *con) {
 		ds = (data_string *)con->response.headers->data[i];
 		
 		if (ds->value->used && ds->key->used &&
-		    0 != strncmp(ds->key->ptr, "X-LIGHTTPD-", sizeof("X-LIGHTTPD-") - 1)) {
+		    0 != strncmp(ds->key->ptr, "X-LIGHTTPD-", sizeof("X-LIGHTTPD-") - 1) &&
+		    /* headers we send */
+		    !buffer_is_equal_string(ds->key, CONST_STR_LEN("Server")) &&
+		    !buffer_is_equal_string(ds->key, CONST_STR_LEN("Date")) &&
+		    !buffer_is_equal_string(ds->key, CONST_STR_LEN("Transfer-Encoding")) &&
+		    !buffer_is_equal_string(ds->key, CONST_STR_LEN("Connection"))) {
 			BUFFER_APPEND_STRING_CONST(b, "\r\n");
 			buffer_append_string_buffer(b, ds->key);
 			BUFFER_APPEND_STRING_CONST(b, ": ");
@@ -147,6 +151,9 @@ int http_response_write_header(server *srv, connection *con,
 	
 	if (con->response.transfer_encoding & HTTP_TRANSFER_ENCODING_CHUNKED) {
 		BUFFER_APPEND_STRING_CONST(b, "\r\nTransfer-Encoding: chunked");
+	} else if (file_size >= 0 && con->http_status != 304) {
+		BUFFER_APPEND_STRING_CONST(b, "\r\nContent-Length: ");
+		buffer_append_off_t(b, file_size);
 	}
 	
 	/* HTTP/1.1 requires a Date: header */
@@ -204,11 +211,6 @@ int http_response_write_header(server *srv, connection *con,
 		buffer_append_string_buffer(b, srv->mtime_cache[i].str);
 	}
 	
-	if (file_size >= 0 && con->http_status != 304) {
-		BUFFER_APPEND_STRING_CONST(b, "\r\nContent-Length: ");
-		buffer_append_off_t(b, file_size);
-	}
-	
 	if (con->physical.path->used && con->physical.etag->used) {
 		BUFFER_APPEND_STRING_CONST(b, "\r\nETag: ");
 		buffer_append_string_buffer(b, con->physical.etag);
@@ -847,10 +849,109 @@ static int http_list_directory(server *srv, connection *con, buffer *dir) {
 }
 
 
+int http_response_handle_cachable(server *srv, connection *con, time_t mtime) {
+	if (con->http_status != 0) return 0;
+
+	/*
+	 * 14.26 If-None-Match
+	 *    [...]
+	 *    If none of the entity tags match, then the server MAY perform the
+	 *    requested method as if the If-None-Match header field did not exist,
+	 *    but MUST also ignore any If-Modified-Since header field(s) in the
+	 *    request. That is, if no entity tags match, then the server MUST NOT
+	 *    return a 304 (Not Modified) response.
+	 */
+	
+	/* last-modified handling */
+	if (con->request.http_if_none_match) {
+		if (etag_is_equal(con->physical.etag, con->request.http_if_none_match)) {
+			if (con->request.http_method == HTTP_METHOD_GET || 
+			    con->request.http_method == HTTP_METHOD_HEAD) {
+				
+				/* check if etag + last-modified */
+				if (con->request.http_if_modified_since) {
+					char buf[64];
+					struct tm tm;
+					size_t used_len;
+					char *semicolon;
+				
+					strftime(buf, sizeof(buf)-1, "%a, %d %b %Y %H:%M:%S GMT", gmtime(&mtime));
+					
+					if (NULL == (semicolon = strchr(con->request.http_if_modified_since, ';'))) {
+						used_len = strlen(con->request.http_if_modified_since);
+					} else {
+						used_len = semicolon - con->request.http_if_modified_since;
+					}
+					
+					if (0 == strncmp(con->request.http_if_modified_since, buf, used_len)) {
+						con->http_status = 304;
+						return 1;
+					} else {
+						/* convert to timestamp */
+						if (used_len < sizeof(buf) - 1) {
+							time_t t;
+							strncpy(buf, con->request.http_if_modified_since, used_len);
+							buf[used_len] = '\0';
+							
+							strptime(buf, "%a, %d %b %Y %H:%M:%S GMT", &tm);
+							
+							if (-1 != (t = mktime(&tm)) &&
+							    t <= mtime) {
+								con->http_status = 304;
+								return 1;
+							}
+						} else {
+							log_error_write(srv, __FILE__, __LINE__, "ss", 
+									con->request.http_if_modified_since, buf);
+							
+							con->http_status = 412;
+							return 1;
+						}
+					}
+				} else {
+					con->http_status = 304;
+					return 1;
+				}
+			} else {
+				con->http_status = 412;	
+				return 1;
+			}
+		}
+	} else if (con->request.http_if_modified_since) {
+		char buf[64];
+		struct tm *tm;
+		size_t used_len;
+		char *semicolon;
+		
+		tm = gmtime(&(mtime));
+		strftime(buf, sizeof(buf)-1, "%a, %d %b %Y %H:%M:%S GMT", tm);
+		
+		if (NULL == (semicolon = strchr(con->request.http_if_modified_since, ';'))) {
+			used_len = strlen(con->request.http_if_modified_since);
+		} else {
+			used_len = semicolon - con->request.http_if_modified_since;
+		}
+		
+		if (0 == strncmp(con->request.http_if_modified_since, buf, used_len)) {
+			con->http_status = 304;
+			return 1;
+		}
+	}
+
+	return 0;
+}
 
 handler_t http_response_prepare(server *srv, connection *con) {
 	handler_t r;
 	
+	if (con->loops_per_request++ > 1000) {
+		/* protect us again endless loops in a single request */
+		
+		log_error_write(srv, __FILE__, __LINE__,  "s",  "ENDLESS LOOP DETECTED ... aborting request");
+		
+		return HANDLER_ERROR;
+	}
+	
 	/* looks like someone has already done a decision */
 	if (con->mode == DIRECT && 
 	    (con->http_status != 0 && con->http_status != 200)) {
@@ -861,6 +962,13 @@ handler_t http_response_prepare(server *srv, connection *con) {
 		
 		return HANDLER_FINISHED;
 	}
+
+	if (con->request.http_method == HTTP_METHOD_OPTIONS) {
+		con->file_finished = 1;
+		con->file_started = 1;
+
+		return HANDLER_FINISHED;
+	}
 	
 	/* no decision yet, build conf->filename */
 	if (con->mode == DIRECT && con->physical.path->used == 0) {
@@ -1180,7 +1288,7 @@ handler_t http_response_prepare(server *srv, connection *con) {
 				 * shorten uri.path
 				 */
 				
-				con->uri.path->used -= strlen(pathinfo);
+				con->uri.path->used -= con->request.pathinfo->used - 1;
 				con->uri.path->ptr[con->uri.path->used - 1] = '\0';
 			}
 			
@@ -1338,89 +1446,12 @@ handler_t http_response_prepare(server *srv, connection *con) {
 			
 			/* generate e-tag */
 			etag_mutate(con->physical.etag, con->fce->etag);
-			
-			/*
-			 * 14.26 If-None-Match
-			 *    [...]
-			 *    If none of the entity tags match, then the server MAY perform the
-			 *    requested method as if the If-None-Match header field did not exist,
-			 *    but MUST also ignore any If-Modified-Since header field(s) in the
-			 *    request. That is, if no entity tags match, then the server MUST NOT
-			 *    return a 304 (Not Modified) response.
-			 */
-			
-			/* last-modified handling */
-			if (con->http_status == 0 && 
-			    con->request.http_if_none_match) {
-				if (etag_is_equal(con->physical.etag, con->request.http_if_none_match)) {
-					if (con->request.http_method == HTTP_METHOD_GET || 
-					    con->request.http_method == HTTP_METHOD_HEAD) {
+
+			http_response_handle_cachable(srv, con, con->fce->st.st_mtime);
 						
-						/* check if etag + last-modified */
-						if (con->request.http_if_modified_since) {
-							char buf[64];
-							struct tm tm;
-							size_t used_len;
-							char *semicolon;
-						
-							strftime(buf, sizeof(buf)-1, "%a, %d %b %Y %H:%M:%S GMT", gmtime(&(con->fce->st.st_mtime)));
-							
-							if (NULL == (semicolon = strchr(con->request.http_if_modified_since, ';'))) {
-								used_len = strlen(con->request.http_if_modified_since);
-							} else {
-								used_len = semicolon - con->request.http_if_modified_since;
-							}
-							
-							if (0 == strncmp(con->request.http_if_modified_since, buf, used_len)) {
-								con->http_status = 304;
-							} else {
-								/* convert to timestamp */
-								if (used_len < sizeof(buf) - 1) {
-									time_t t;
-									strncpy(buf, con->request.http_if_modified_since, used_len);
-									buf[used_len] = '\0';
-									
-									strptime(buf, "%a, %d %b %Y %H:%M:%S GMT", &tm);
-									
-									if (-1 != (t = mktime(&tm)) &&
-									    t <= con->fce->st.st_mtime) {
-										con->http_status = 304;
-									}
-								} else {
-									log_error_write(srv, __FILE__, __LINE__, "ss", 
-											con->request.http_if_modified_since, buf);
-									
-									con->http_status = 412;
-								}
-							}
-						} else {
-							con->http_status = 304;
-						}
-					} else {
-						con->http_status = 412;
-					}
-				}
-			} else if (con->http_status == 0 && con->request.http_if_modified_since) {
-				char buf[64];
-				struct tm *tm;
-				size_t used_len;
-				char *semicolon;
-				
-				tm = gmtime(&(con->fce->st.st_mtime));
-				strftime(buf, sizeof(buf)-1, "%a, %d %b %Y %H:%M:%S GMT", tm);
-				
-				if (NULL == (semicolon = strchr(con->request.http_if_modified_since, ';'))) {
-					used_len = strlen(con->request.http_if_modified_since);
-				} else {
-					used_len = semicolon - con->request.http_if_modified_since;
-				}
-				
-				if (0 == strncmp(con->request.http_if_modified_since, buf, used_len)) {
-					con->http_status = 304;
-				}
-			}
-			
-			if (con->http_status == 0 && con->request.http_range) {
+			if (con->conf.range_requests &&
+			    con->http_status == 0 && 
+			    con->request.http_range) {
 				http_response_parse_range(srv, con);
 			} else if (con->http_status == 0) {
 				switch(r = plugins_call_handle_physical_path(srv, con)) {
